# -*- coding: utf-8 -*-

from __future__ import absolute_import

from seisma import Seisma

from ... import reason
from ...utils import pyv
from ... import layers as _layers


CONFIG_KEY = 'SEISMA_EX'

DEFAULT_JOB_NAME = 'seismograph'
DEFAULT_JOB_TITLE = 'Testing with seismograph library'


def _get_doc_from_object(obj):
    doc = getattr(obj, '__doc__', None)

    if doc:
        if pyv.IS_PYTHON_2:
            return doc.decode('utf-8').strip()
        return doc.strip()

    return None


def _get_client_kwargs_from_config(config):
    params = config.get(CONFIG_KEY, {})

    base_url = config.SEISMA_URL or params.get('URL')
    assert base_url, '"URL" to seisma is not configured'

    job_name = params.get('JOB_NAME', DEFAULT_JOB_NAME)
    job_title = params.get('JOB_TITLE', DEFAULT_JOB_TITLE)

    build_title = config.SEISMA_BUILD_TITLE or params.get('BUILD_TITLE')

    api_version = params.get('API_VERSION')
    build_name = config.SEISMA_BUILD_NAME or params.get('BUILD_NAME')
    build_metadata = params.get('BUILD_METADATA')

    return {
        'base_url': base_url,
        'job_name': job_name,
        'job_title': job_title,
        'build_name': build_name,
        'build_title': build_title,
        'api_version': api_version,
        'build_metadata': build_metadata,
    }


def _get_reason(obj, tb):
    return reason.format_reason(
        reason.create(
            obj, tb, config=obj.config,
        ),
    )


def _get_case_name(case):
    return '{}:{}'.format(
        case.__mount_data__.suite_name,
        case.__class__.__name__
    )


class SeismaProgramLayer(_layers.ProgramLayer):

    client = None

    def on_init(self, program):
        if program.config.SEISMA:
            self.client = Seisma(
                **_get_client_kwargs_from_config(program.config)
            )
        else:
            self.enabled = False

    def on_setup(self, program):
        if not self.client.job_exists():
            kwargs = {}
            doc = _get_doc_from_object(program)

            if doc:
                kwargs['description'] = doc

            self.client.create_job(**kwargs)

        self.client.start_build()

    def on_teardown(self, program):
        kwargs = {
            'runtime': program.result.current_state.runtime,
            'tests_count': program.result.current_state.tests,
            'error_count': program.result.current_state.errors,
            'fail_count': program.result.current_state.failures,
            'was_success': program.result.current_state.was_success,
            'success_count': program.result.current_state.successes,
        }

        self.client.stop_build(**kwargs)


class SeismaCaseLayer(_layers.CaseLayer):

    client = None

    def on_init(self, case):
        if case.config.SEISMA:
            self.client = Seisma(
                **_get_client_kwargs_from_config(case.config)
            )
        else:
            self.enabled = False

    def on_setup(self, case):
        case_name = _get_case_name(case)

        if not self.client.case_exists_on_job(case_name):
            kwargs = {
                'name': case_name,
            }
            doc = _get_doc_from_object(case)

            if doc:
                kwargs['description'] = doc

            self.client.add_case_to_job(**kwargs)

    def on_success(self, case, timer):
        kwargs = {
            'name': _get_case_name(case),
            'status': 'passed',
            'runtime': timer(),
        }
        metadata = getattr(case, 'metadata', None)

        if isinstance(metadata, dict):
            kwargs['metadata'] = metadata

        self.client.add_case_result(**kwargs)

    def on_fail(self, fail, case, result, tb, timer):
        kwargs = {
            'name': _get_case_name(case),
            'status': 'failed',
            'runtime': timer(),
            'reason': _get_reason(case, tb),
        }
        metadata = getattr(case, 'metadata', None)

        if isinstance(metadata, dict):
            kwargs['metadata'] = metadata

        self.client.add_case_result(**kwargs)

    def on_any_error(self, error, case, result, tb, timer):
        kwargs = {
            'name': _get_case_name(case),
            'status': 'error',
            'runtime': timer(),
            'reason': _get_reason(case, tb),
        }
        metadata = getattr(case, 'metadata', None)

        if isinstance(metadata, dict):
            kwargs['metadata'] = metadata

        self.client.add_case_result(**kwargs)

    def on_skip(self, case, reason, result):
        # If case was skipped and not created
        # for seisma server we should create
        # it because 404 otherwise
        self.on_setup(case)

        params = {
            'name': _get_case_name(case),
            'status': 'skipped',
            'runtime': 0.0,
            'reason': reason,
        }
        metadata = getattr(case, 'metadata', None)

        if isinstance(metadata, dict):
            params['metadata'] = metadata

        self.client.add_case_result(**params)



# -*- coding: utf-8 -*-

from optparse import OptionGroup

from .layers import SeismaCaseLayer
from .layers import SeismaProgramLayer


__all__ = (
    'SeismaCaseLayer',
    'SeismaProgramLayer',
)


def __add_options__(parser):
    group = OptionGroup(parser, 'Seisma extension options')

    group.add_option(
        '--seisma',
        dest='SEISMA',
        action='store_true',
        default=False,
        help='Use aggregation analytics to seisma.'
    )
    group.add_option(
        '--seisma-url',
        dest='SEISMA_URL',
        default=None,
        help='Base URL to seisma.',
    )
    group.add_option(
        '--seisma-build-name',
        dest='SEISMA_BUILD_NAME',
        default=None,
        help='Unique build name.',
    )
    group.add_option(
        '--seisma-build-title',
        dest='SEISMA_BUILD_TITLE',
        default=None,
        help='Title of build.',
    )

    parser.add_option_group(group)


def __install__(program):
    if program.config.SEISMA:
        from seismograph import case as c
        from seismograph import program as p

        c.DEFAULT_LAYERS.append(
            SeismaCaseLayer(),
        )
        p.DEFAULT_LAYERS.append(
            SeismaProgramLayer(),
        )



# -*- coding: utf-8 -*-

from . import runnable


class CaseLayer(runnable.LayerOfRunnableObject):

    def on_init(self, case):
        """
        :type case: Case
        """
        pass

    def on_require(self, require):
        """
        :type require: list
        """
        pass

    def on_setup(self, case):
        """
        :type case: Case
        """
        pass

    def on_teardown(self, case):
        """
        :type case: Case
        """
        pass

    def on_skip(self, case, reason, result):
        """
        :type case: Case
        :type reason: str
        :type result: seismograph.result.Result
        """
        pass

    def on_any_error(self, error, case, result, tb, timer):
        """
        :type case: Case
        :type error: BaseException
        :type result: seismograph.result.Result
        :type tb: str
        :type timer: callable
        """
        pass

    def on_error(self, error, case, result, tb, timer):
        """
        :type case: Case
        :type error: BaseException
        :type result: seismograph.result.Result
        :type tb: str
        :type timer: callable
        """
        pass

    def on_context_error(self, error, case, result, tb, timer):
        """
        :type case: Case
        :type error: BaseException
        :type result: seismograph.result.Result
        :type tb: str
        :type timer: callable
        """
        pass

    def on_fail(self, fail, case, result, tb, timer):
        """
        :type case: Case
        :type fail: AssertionError
        :type result: seismograph.result.Result
        :type tb: str
        :type timer: callable
        """
        pass

    def on_success(self, case, timer):
        """
        :type case: Case
        """
        pass

    def on_run(self, case):
        """
        :type case: Case
        """
        pass


class SuiteLayer(runnable.LayerOfRunnableObject):

    def on_init(self, suite):
        """
        :type suite: Suite
        """
        pass

    def on_require(self, require):
        """
        :type require: list
        """
        pass

    def on_build_rule(self, suite, rule):
        """
        :type suite: Suite
        :type rule: BuildRule
        """
        pass

    def on_setup(self, suite):
        """
        :type suite: Suite
        """
        pass

    def on_teardown(self, suite):
        """
        :type suite: Suite
        """
        pass

    def on_mount(self, suite, program):
        """
        :type suite: Suite
        :type program: seismograph.program.Program
        """
        pass

    def on_run(self, suite):
        """
        :type suite: Suite
        """
        pass

    def on_error(self, error, suite, result, tb, timer):
        """
        :type error: BaseException
        :type suite: Suite
        :type result: seismograph.result.Result
        :type tb: str
        :type timer: callable
        """
        pass


class ProgramLayer(runnable.LayerOfRunnableObject):

    def on_init(self, program):
        """
        :type program: Program
        """
        pass

    def on_config(self, config):
        """
        :type config: seismograph.config.Config
        """
        pass

    def on_setup(self, program):
        """
        :type program: Program
        """
        pass

    def on_teardown(self, program):
        """
        :type program: Program
        """
        pass

    def on_error(self, error, program, result, tb, timer):
        """
        :type error: BaseException
        :type program: Program
        :type result: seismograph.result.Result
        :type tb: str
        :type timer: callable
        """
        pass

    def on_option_parser(self, parser):
        """
        :param parser: optparse.OptionParser
        """
        pass

    def on_run(self, program):
        """
        :type program: Program
        """
        pass



# -*- coding: utf-8 -*-

import sys
import time
from . import pyv

from ..exceptions import TimeoutException


def waiting_for(func, timeout=None, exc_cls=None, message=None, delay=None, args=None, kwargs=None):
    args = args or tuple()
    kwargs = kwargs or dict()

    timeout = timeout or 0
    message = message or 'Timeout "{}" exceeded'.format(timeout)

    if timeout:
        t_start = time.time()

        while time.time() <= t_start + timeout:
            result = func(*args, **kwargs)

            if result:
                return result

            if delay:
                time.sleep(delay)
        else:
            if exc_cls:
                raise exc_cls(message)
            raise TimeoutException(message)

    result = func(*args, **kwargs)

    if result:
        return result
    if exc_cls:
        raise exc_cls(message)
    raise TimeoutException(message)


def call_to_chain(chain, method_name, *args, **kwargs):
    for obj in chain:
        if method_name:
            getattr(obj, method_name)(*args, **kwargs)
        else:
            obj(*args, **kwargs)


def measure_time():
    start_time = time.time()
    return lambda: time.time() - start_time


def pythonpaths(*paths):
    def wrapper(f):
        for path in paths:
            if path not in sys.path:
                sys.path.append(path)
        return f
    return wrapper


def get_dict_from_list(lst, **kwargs):
    """
    Get dictionary by equal filters from kwargs

    Example:

        lst = [
            {
                'name': 'hello',
            },
            {
                'name': 'world',
            }
        ]

        dct = get_dict_from_list(lst, name='hello')
    """
    if kwargs and isinstance(lst, (list, tuple)):
        for item in lst:
            comparison = all(
                item.get(k) == v
                if isinstance(v, (int, pyv.basestring))
                else
                True
                for k, v in kwargs.items()
            )
            if comparison:
                return item
        else:
            raise LookupError(
                'Dictionary from list is not found by filters: {}'.format(kwargs),
            )

    return lst


def reduce_dict(d1, d2):
    """
    Leads dictionaries to the same species.
    The standard dictionary is d2.
    """
    def prepare_lists(l1, l2):
        assert len(l1) == len(l2)

        for i in l1:
            if isinstance(i, dict):
                l1[l1.index(i)] = reduce_dict(i, l2[l1.index(i)])

        l1.sort()
        l2.sort()

    return dict(
        (
            k,
            reduce_dict(v, d2[k])
            if isinstance(v, dict) and isinstance(d2[k], dict)
            else
            prepare_lists(v, d2[k]) or v
            if isinstance(v, list) and isinstance(d2[k], list)
            else
            v,
        )
        for k, v in d1.items()
        if k in d2
    )


def reduce_list(l1, l2):
    """
    Do return new lists. l2 is standard list.
    Equal length of lists is important.
    """
    assert len(l1) == len(l2), 'Length of lists is not equal: {} != {}'.format(len(l1), len(l2))

    lst = []

    for i in l1:
        v1 = i
        v2 = l2[l1.index(i)]

        if isinstance(v1, dict):
            lst.append(reduce_dict(v1, v2))
        elif isinstance(v1, (list, tuple)):
            lst.append(reduce_list(v1, v2))
        else:
            lst.append(v1)

    lst.sort()
    l2.sort()

    return lst, list(l2)
